#include <stdio.h>
#include <math.h>
/* prepara_sencos.c : file sorgente di un programma che crea un file
                      "calc_sencos.inp" nel quale sono riportati
		      i valori del seno e del coseno degli angoli
		          (i * pigreco) / (2 * POT2FIN)
		      dove i=0,1,...,POT2FIN.
                      I suddetti valori sono calcolati senza far
                      ricorso a funzioni trigonometriche, ma facendo
                      uso di operazioni che sono tutte calcolabili
                      con un errore all'ultimo bit significativo */
/* ### */
/* La prossima definizione consente di cambiare facilmente il numero
   totale di seni e coseni calcolati (POT2FIN gioca il ruolo di N nel
   testo del tema d'esame) */
#define POT2FIN 8
/* ### */
int main()
{
  int i, pot2;
  double vet_seni[POT2FIN+1], vet_coseni[POT2FIN+1];
  FILE *ofp;
  /* Definizioni iniziali del calcolo iterativo delle successioni
     S_N e C_N (vedi testo del tema d'esame).
     Poniamo pot2 (che gioca il ruolo di n) uguale a 1 ... */
  pot2 = 1;
  /* ... Definiamo il vettore vet_seni in modo che i suoi elementi
     coincidano quelli con quelli di S_1 */
  vet_seni[0] = 0.;  vet_seni[1] = 1.;
  /* ... e il vettore vet_coseni in modo che i suoi elementi
     coincidano quelli con quelli di C_1 */
  vet_coseni[0] = 1.;  vet_coseni[1] = 0.;
  /* A questo punto dobbiamo eseguire il calcolo di S_2n e C_2n a
     partire da S_n e C_n (nel modo descritto sul testo del tema
     d'esame), dove n all'inizio e' uguale a 1 e alla fine e' uguale a
     POT2FIN/2.
     A questo scopo eseguiamo il ciclo di istruzioni seguenti mentre e'
     verificata la condizione che pot2 sia minore di POT2FIN */
  while(pot2 < POT2FIN) {
    /* Eseguiamo un ciclo di istruzioni mentre il contatore i va da
       pot2 a 1 */
    for(i=pot2; i>0; i--) {
      /* A questo punto dobbiamo definire l'elemento 2i-esimo di S_2n
	 (ricordando che S_2n e' rappresentato dal vettore vet_seni)
	 in modo tale che sia uguale all'elemento i-esimo di S_n
	 (che e' anch'esso rappresentato dal vettore vet_seni) */
      vet_seni[2*i] = vet_seni[i];
      /* Ora dobbiamo definire l'elemento 2i-esimo di C_2n (ricordando
	 che C_2n e' rappresentato dal vettore vet_coseni) in modo
	 tale che sia uguale all'elemento i-esimo di C_n (che e'
	 anch'esso rappresentato dal vettore vet_coseni) */
      vet_coseni[2*i] = vet_coseni[i];
    }
    /* Eseguiamo un ciclo di istruzioni mentre il contatore i assume tutti
       i valori dispari compresi tra 1 e pot2 */
    for(i=1; i<=pot2; i+=2) {
      /* Definiamo l'elemento i-esimo di C_2n (cioe' di vet_coseni)
	 a partire dall'elemento 2i-esimo di C_2n, sfruttando
	 la formula di bisezione del coseno (e' quella con la radice
	 quadrata riportata nel tema d'esame) */
      vet_coseni[i] = sqrt( (1.+vet_coseni[2*i]) / 2.);
      /* Definiamo l'elemento i-esimo di S_2n (cioe' di vet_seni) come
	 la meta' del rapporto tra l'elemento 2i-esimo di S_2n e
	 quello i-esimo di C_2n */
      vet_seni[i] = 0.5 * vet_seni[2*i] / vet_coseni[i];
      /* Definiamo l'elemento 2n-i-esimo (si ricordi che pot2 gioca il
	 ruolo di n) di S_2n uguale all'elemento i-esimo di C_2n */
      vet_seni[2*pot2-i] = vet_coseni[i];
      /* Definiamo l'elemento 2n-i-esimo di C_2n uguale all'elemento
	 i-esimo di S_2n */
      vet_coseni[2*pot2-i] = vet_seni[i];
    }
    /* Raddoppiamo il valore di pot2 */
    pot2 *= 2;
  }
  /* Apriamo in modalita' di scrittura il file di output che si chiama
     "calc_sencos.inp", associamo a ofp l'indirizzo corrispondente al
     file */
  ofp = fopen("calc_sencos.inp", "w");
  /* Scriviamo i valori della variabile pot2 (che a questo punto e'
     sicuramente uguale a POT2FIN) sul file che si chiama
     "calc_sencos.inp".
     (Consiglio: si scriva in modo binario o ASCII in modo coerente
      a quanto si fara' poche istruzioni sotto. Al fine di scegliere
      la modalita' di scrittura si leggano i consigli poco sotto) */
  fwrite(&pot2,sizeof(int),1,ofp);
  /* Eseguiamo il seguente ciclo di istruzioni mentre il contatore i
     va da 0 a POT2FIN */
  for(i=0; i<=POT2FIN; i++) {
    /* Scriviamo i valori delle variabili vet_seni[i] e vet_coseni[i]
       sul file che si chiama "calc_sencos.inp".
       (Consiglio 1: e' meglio scrivere i valori delle variabili in
        modo binario, utilizzando la funzione fwrite. In tal modo
	siamo sicuri di non perdere informazioni durante la
	conversione in ASCII nell'operazione di scrittura e nella
	conseguente lettura.
	Consiglio 2: se fwrite risulta proprio indigesta, si puo'
	comunque utilizzare fprintf ...) */
    fwrite(&vet_seni[i],sizeof(double),1,ofp);
    fwrite(&vet_coseni[i],sizeof(double),1,ofp);
    /* Al fine di controllare che i calcoli siano stati effettuati
       correttamente, scriviamo su video i valori delle variabili
       vet_seni[i] e vet_coseni[i].
       (Consiglio: al fine di poter fare dei controlli accurati, si
        scrivano questi valori con molte cifre significative) */
    printf("%24.16le %24.16le\n", vet_seni[i], vet_coseni[i]);
  }
  /* Chiudiamo il file che si chiama "calc_sencos.inp" */
  fclose(ofp);
  exit(0);
}
