#include <stdio.h>
#include <math.h>
/* polari.c : file sorgente di un programma che comprende due funzioni
              che calcolano, rispettivamente, la trasformazione di
              coordinate da cartesiane a polari e la sua inversa.
	      Le suddette funzioni vengono utilizzate per verificare
	      le formule delle equazioni dell'ellisse e della parabola
	      in coordinate polari. I valori degli errori nei tests,
	      vengono scritti in un file di nome "polari.out" che
	      verra' successivamente analizzato dal programma nel file
	      err_max_polari.c */
/* ### */
/* La prossima definizione consente di cambiare facilmente il numero
   massimo di tests che vengono calcolati */
#define NTEST 10
/* Definizione dei prototipi delle funzioni */
void da_cartesiane_a_polari(double x, double y, double *rho, double *theta);
void da_polari_a_cartesiane(double rho, double theta, double *x, double *y);
/* ### */
int main()
{
  int i;
  double x, y, rho, theta;
  double duepigreco, p, e, a, b, x0, err;
  FILE *ofp;
  /* Definizione di duepigreco */
  duepigreco = 8. * atan(1.);
  /* Apriamo in modalita' di scrittura il file di output che si chiama
     "polari.out", associamo a ofp l'indirizzo corrispondente al file */
  ofp = fopen("polari.out", "w");
  /* Scriviamo in cima al file "polari.out" il numero di tests che
     effettueremo */
  fprintf(ofp, "%3d\n", NTEST);
  /* Fase di input del parametro p */
  do {
    printf("Parametro p dell'ellisse? (p deve essere > 0)\n");
    scanf("%lf", &p);
  } while(p <= 0.);
  /* Fase di input dell'eccentricita' e */
  do {
    printf("Eccentricita' e dell'ellisse? (deve essere 0 < e < 1)\n");
    scanf("%lf", &e);
  } while(e <= 0. || e >= 1.);
  /* Calcolo delle quantita' a, b e x0 a partire da p ed e */
  a = p / (1. - e * e);
  b = p / sqrt(1. - e * e);
  x0 = - e * a;
  printf("  Test sull'equazione dell'ellisse:\n");
  printf("#test  errore\n");
  for(i=0; i<NTEST; i++) {
    theta = (i * duepigreco) / NTEST;
    rho = p / (1. + e * cos(theta));
    da_polari_a_cartesiane(rho, theta, &x, &y);
    err = fabs( (x-x0)*(x-x0)/(a*a) + y*y/(b*b) - 1. );
    printf("%3d    %le\n", i, err);
    /* Stampa dell'errore sul file "polari.out" */
    fprintf(ofp, "%le\n", err);
  }
  /* Fase di input del coefficiente a della parabola */
  do {
    printf("Coefficiente a della parabola x = - a*y^2 + 1/(4*a) ? ");
    printf("(a deve essere > 0)\n");
    scanf("%lf", &a);
  } while(a <= 0.);
  p = 1. / (2. * a);
  /* Calcolo delle quantita' a, b e x0 a partire da p ed e */
  printf("  Test sull'equazione della parabola:\n");
  printf("#test  errore\n");
  for(i=0; i<NTEST; i++) {
    y = (2. * i) / (a * (NTEST - 1) ) - 1. / a;
    x = -a*y*y + 1./(4.*a);
    da_cartesiane_a_polari(x, y, &rho, &theta);
    err = fabs( 1. - p / (rho * (1 + cos(theta))) );
    printf("%3d    %le\n", i, err);
    /* Stampa dell'errore sul file "polari.out" */
    fprintf(ofp, "%le\n", err);
  }
  /* Chiusura del file "polari.out" */
  fclose(ofp);
  exit(0);
}
/* ### */
void da_cartesiane_a_polari(x, y, rho, theta)
     double x, y, *rho, *theta;
{
  *rho = sqrt(x*x + y*y);
  *theta = atan2(y, x);
}
/* ### */
void da_polari_a_cartesiane(rho, theta, x, y)
     double rho, theta, *x, *y;
{
  *x = rho * cos(theta);
  *y = rho * sin(theta);
}
